import subprocess  # 导入 subprocess 模块，用于执行系统命令
import re  # 导入 re 模块，用于正则表达式操作

# 定义 URL 和成功关键字列表
URL = 'https://eapi.jiasu123.net/bs/bdCoSpeed/deaab7633f2c636410b50f3be3b80f2a8190efa82e9f11141c12efcded2c841ff9f4e2d6ef420479f2200a02704a0393'
SUCCESS_KEYWORDS = ["正在", "成功", "服务", "重复", "开始提速"]
PPP_CURL_FILE = '/root/ppp-curl.txt'  # 定义存储接口和 IP 的文件路径

def run_curl_task(interface):
    """运行 curl 任务并检查结果"""
    try:
        print(f"Running curl on interface {interface}...")  # 打印当前正在运行的接口
        # 使用指定接口运行 curl 命令
        result = subprocess.check_output(['curl', '--interface', interface, URL], stderr=subprocess.STDOUT).decode('utf-8')
        print(f"curl result on {interface}: {result}")  # 打印 curl 命令的结果
        # 检查结果中是否包含任意一个成功关键字
        if any(keyword in result for keyword in SUCCESS_KEYWORDS):
            print(f"Task succeeded on interface {interface}")  # 打印任务成功信息
            remove_line_from_file(interface, '/root/ppp-curl.txt') 
            return True
        else:
            print(f"Task failed on interface {interface}, killing process...")  # 打印任务失败信息
            kill_ppp_process(interface)  # 终止相关进程
            return False
    except subprocess.CalledProcessError as e:
        print(f"Error running curl task on {interface}: {e.output.decode('utf-8')}")  # 打印错误信息
        kill_ppp_process(interface)  # 终止相关进程
        return False

def kill_ppp_process(interface):
    """查询 ppp 进程并 kill"""
    try:
        print(f"Killing process for interface {interface}...")  # 打印正在终止的接口信息
        # 提取接口编号
        interface_number = interface.lstrip('ppp')
        output = subprocess.check_output(['ps', '-aux']).decode('utf-8')  # 获取所有进程信息
        lines = output.split('\n')  # 按行分割进程信息
        for line in lines:
            # 匹配 pppd 进程
            pattern = re.compile(rf'pppd call ppp{interface_number}\b')
            if pattern.search(line) and 'grep' not in line:
                process_id = re.split(r'\s+', line)[1]  # 提取进程 ID
                print(f"Found process ID {process_id} for interface {interface}")  # 打印找到的进程 ID
                subprocess.run(['kill', '-9', process_id], check=True)  # 强制终止进程
                print(f"Process {process_id} for interface {interface} killed")  # 打印进程终止信息
                break
        else:
            print(f"No process found for interface {interface}")  # 打印未找到进程信息
    except subprocess.CalledProcessError as e:
        print(f"Error killing process for interface {interface}: {e}")  # 打印错误信息

def main():
    try:
        with open(PPP_CURL_FILE, 'r') as file:  # 打开存储接口和 IP 的文件
            for line in file:
                interface, ip = line.strip().split()  # 读取每一行并分割为接口和 IP
                if not run_curl_task(interface):  # 运行 curl 任务并检查结果
                    print("Curl task failed.")  # 打印任务失败信息
    except FileNotFoundError:
        print(f"Error: {PPP_CURL_FILE} not found.")  # 打印文件未找到错误信息

if __name__ == "__main__":
    main()  # 运行主函数
